@@ -40,7 +40,7 @@ enum client_state
     Client_Disconnected
 };
 
-int clients_count(void);
+int clients_get_count(void);
 int client_show_raw_begin(struct client *cl, int position,
                           const char *syntax, const char *esn,
                           void *data,
@@ -69,6 +69,7 @@ int client_destroy(struct client *c);
 
 void client_set_connection(struct client *cl, struct connection *con);
 void client_disconnect(struct client *cl);
+void client_mark_dead(struct client *cl);
 int client_prep_connection(struct client *cl,
                            int operation_timeout, int session_timeout,
                            iochan_man_t iochan,
@@ -77,7 +78,8 @@ int client_start_search(struct client *cl);
 int client_fetch_more(struct client *cl);
 int client_parse_init(struct client *cl, int same_search);
 int client_parse_range(struct client *cl, const char *startrecs, const char *maxrecs);
-int client_parse_sort(struct client *cl, struct reclist_sortparms *sp);
+int client_parse_sort(struct client *cl, struct reclist_sortparms *sp,
+                      int *has_sortmap);
 void client_set_session(struct client *cl, struct session *se);
 int client_is_active(struct client *cl);
 int client_is_active_preferred(struct client *cl);
@@ -104,17 +106,20 @@ void client_unlock(struct client *c);
 void client_stop(struct client *c);
 
 int client_has_facet(struct client *cl, const char *name);
-void client_check_preferred_watch(struct client *cl);
 int client_reingest(struct client *cl);
 const char *client_get_facet_limit_local(struct client *cl,
                                          struct session_database *sdb,
                                          int *l,
                                          NMEM nmem, int *num, char ***values);
 
+const char *client_get_suggestions_xml(struct client *cl, WRBUF wrbuf);
+
 void client_update_show_stat(struct client *cl, int cmd);
 
 void client_store_xdoc(struct client *cl, int record_no, xmlDoc *xdoc);
 
+const char *client_get_query(struct client *cl, const char **type, NMEM nmem);
+
 #endif
 
 /*
