@@ -742,12 +742,12 @@ static int is_inprogress(void)
 
 static void enable_nonblock(int sock)
 {
-    int flags;
 #ifdef WIN32
-    flags = (flags & CS_FLAGS_BLOCKING) ? 0 : 1;
+    unsigned long flags = 1;
     if (ioctlsocket(sock, FIONBIO, &flags) < 0)
         yaz_log(YLOG_FATAL|YLOG_ERRNO, "ioctlsocket");
 #else
+    int flags;
     if ((flags = fcntl(sock, F_GETFL, 0)) < 0)
         yaz_log(YLOG_FATAL|YLOG_ERRNO, "fcntl");
     if (fcntl(sock, F_SETFL, flags | O_NONBLOCK) < 0)
@@ -914,10 +914,10 @@ static void http_io(IOCHAN i, int event)
                     fprintf(hc->http_server->record_file, "r %lld %lld %lld 0\n",
                             (long long) tv.tv_sec, (long long) tv.tv_usec,
                             (long long) iochan_getfd(i));
+                    fflush(hc->http_server->record_file);
                 }
 #endif
                 http_buf_destroy(hc->http_server, htbuf);
-                fflush(hc->http_server->record_file);
                 http_channel_destroy(i);
                 return;
             }
@@ -1226,6 +1226,7 @@ static void http_accept(IOCHAN i, int event)
     ch = http_channel_create(server->http_server, host, server);
     ch->iochan = c;
     iochan_setdata(c, ch);
+    iochan_settimeout(c, 60);
     if (iochan_add(server->iochan_man, c, 0))
     {
         yaz_log(YLOG_WARN, "Refusing incoming HTTP connection");
@@ -1343,6 +1344,9 @@ int http_init(struct conf_server *server, const char *record_fname)
     if (iochan_add(server->iochan_man, c, 0))
     {
         yaz_log(YLOG_WARN, "Can not create HTTP binding socket");
+        CLOSESOCKET(s);
+        if (record_file)
+            fclose(record_file);
         iochan_destroy(c);
         return -1;
     }
