@@ -71,14 +71,13 @@ struct session_database
 #define SESSION_WATCH_BYTARGET  4
 #define SESSION_WATCH_MAX       4
 
-#define SESSION_MAX_TERMLISTS 10
-
 typedef void (*session_watchfun)(void *data);
 
 struct named_termlist
 {
     char *name;
     struct termlist *termlist;
+    struct named_termlist *next;
 };
 
 struct session_watchentry {
@@ -97,8 +96,7 @@ struct session {
     struct client_list *clients_cached; // Clients in cache
     NMEM session_nmem;  // Nmem for session-permanent storage
     NMEM nmem;          // Nmem for each operation (i.e. search, result set, etc)
-    int num_termlists;
-    struct named_termlist termlists[SESSION_MAX_TERMLISTS];
+    struct named_termlist *termlists;
     struct relevance *relevance;
     struct reclist *reclist;
     char *mergekey;
@@ -114,7 +112,9 @@ struct session {
     int settings_modified;
     facet_limits_t facet_limits;
     int clients_starting;
+    int force_position;
     struct reclist_sortparms *sorted_results;
+    struct facet_id *facet_id_list;
 };
 
 struct statistics {
@@ -143,16 +143,19 @@ struct hitsbytarget {
     int connected;
     char *settings_xml;
     char *suggestions_xml;
+    const char *query_type;
+    const char *query_data;
 };
 
 struct hitsbytarget *get_hitsbytarget(struct session *s, int *count, NMEM nmem);
-struct session *new_session(NMEM nmem, struct conf_service *service,
-                            unsigned session_id);
-int sessions_count(void);
+struct session *session_create(NMEM nmem, struct conf_service *service,
+                               unsigned session_id);
 void session_destroy(struct session *s);
 void session_init_databases(struct session *s);
 void statistics(struct session *s, struct statistics *stat);
 
+int sessions_get_count(void);
+
 void session_sort(struct session *se, struct reclist_sortparms *sp,
                   const char *mergekey, const char *rank);
 
@@ -192,11 +195,15 @@ int ingest_record(struct client *cl, const char *rec, int record_no, NMEM nmem);
 int ingest_xml_record(struct client *cl, xmlDoc *xdoc,
                       int record_no, NMEM nmem, int cached_copy);
 void session_alert_watch(struct session *s, int what);
-void add_facet(struct session *s, const char *type, const char *value, int count);
+void add_facet(struct session *s, const char *type, const char *value, int count, struct client *cl);
 
 int session_check_cluster_limit(struct session *se, struct record_cluster *rec);
 
 void perform_termlist(struct http_channel *c, struct session *se, const char *name, int num, int version);
+
+const char *session_lookup_id_facet(struct session *s, struct client *cl,
+                                    const char *type, const char *term);
+
 void session_log(struct session *s, int level, const char *fmt, ...)
 #ifdef __GNUC__
     __attribute__ ((format (printf, 3, 4)))
